// shamelessly stolen from the sample MSP430 code from slac288a
//
// modified by Todd Krein

#include "msp430f4132.h"

unsigned char TXData;
unsigned char TXByteCtr;
unsigned char RXData;
unsigned char RXCompare;
unsigned char TXing;

void I2C_WriteByte(unsigned char addr, unsigned char data)
{
  WDTCTL = WDTPW + WDTHOLD;                 // Stop WDT
  P6SEL |= 0x06;                            // Assign I2C pins to USCI_B0
  UCB0CTL1 |= UCSWRST;                      // Enable SW reset
  UCB0CTL0 = UCMST + UCMODE_3 + UCSYNC;     // I2C Master, synchronous mode
  UCB0CTL1 = UCSSEL_2 + UCSWRST;            // Use SMCLK, keep SW reset
  UCB0BR0 = 12;                             // fSCL = SMCLK/12 = ~100kHz
  UCB0BR1 = 0;
  UCB0I2CSA = addr;                    
  UCB0CTL1 &= ~UCSWRST;                     // Clear SW reset, resume operation
  IE2 |= UCB0TXIE;                          // Enable TX interrupt

  TXData = data;                            // Holds TX data
  TXing = 1;								// we are transmitting

  while (1)
  {
    TXByteCtr = 1;                          // Load TX byte counter
    while (UCB0CTL1 & UCTXSTP);             // Ensure stop condition got sent
    UCB0CTL1 |= UCTR + UCTXSTT;             // I2C TX, start condition
    __bis_SR_register(CPUOFF + GIE);        // Enter LPM0 w/ interrupts
                                            // Remain in LPM0 until all data
                                            // is TX'd
    TXData++;                               // Increment data byte
  }
}


void I2C_ReadByte(unsigned char addr, unsigned char *data)
{
  WDTCTL = WDTPW + WDTHOLD;                 // Stop WDT
  P5OUT &= ~0x02;                           // P5.1 =0
  P5DIR |= 0x02;                            // P5.1 output
  P6SEL |= BIT1 + BIT2;                     // Assign I2C pins to USCI_B0
  UCB0CTL1 |= UCSWRST;                      // Enable SW reset
  UCB0CTL0 = UCMST + UCMODE_3 + UCSYNC;     // I2C Master, synchronous mode
  UCB0CTL1 = UCSSEL_2 + UCSWRST;            // Use SMCLK, keep SW reset
  UCB0BR0 = 12;                             // fSCL = SMCLK/12 = ~100kHz
  UCB0BR1 = 0;
  UCB0I2CSA = addr;                      
  UCB0CTL1 &= ~UCSWRST;                     // Clear SW reset, resume operation
  IE2 |= UCB0RXIE;                          // Enable RX interrupt
  RXCompare = 0;                            // Used to check incoming data
  TXing = 0;								// not transmitting

//  while (1)
//  {
    while (UCB0CTL1 & UCTXSTP);             // Ensure stop condition got sent
    UCB0CTL1 |= UCTXSTT;                    // I2C start condition
    while (UCB0CTL1 & UCTXSTT);             // Start condition sent?
    UCB0CTL1 |= UCTXSTP;                    // I2C stop condition
    __bis_SR_register(CPUOFF + GIE);        // Enter LPM0 w/ interrupts
    *data = RXData;

//    if (RXData != RXCompare)                // Trap CPU if wrong
//    {
//      P5OUT |= 0x02;                        // P5.1 = 1
//      while (1);                            // Trap CPU
//    }
//
//    RXCompare++;                            // Increment correct RX value
//  }
}


//------------------------------------------------------------------------------
// The USCIAB0TX_ISR is structured such that it can be used to transmit any
// number of bytes by pre-loading TXByteCtr with the byte count.
//------------------------------------------------------------------------------
#pragma vector = USCIAB0TX_VECTOR
__interrupt void USCIAB0TX_ISR(void)
{
	if (TXing) {
		if (TXByteCtr)                            // Check TX byte counter
		{
			UCB0TXBUF = TXData;                     // Load TX buffer
			TXByteCtr--;                            // Decrement TX byte counter
		}
		else
		{
			UCB0CTL1 |= UCTXSTP;                    // I2C stop condition
			IFG2 &= ~UCB0TXIFG;                     // Clear USCI_B0 TX int flag
			__bic_SR_register_on_exit(CPUOFF);      // Exit LPM0
		}
	}
	else
	{
		RXData = UCB0RXBUF;                       // Get RX data
		__bic_SR_register_on_exit(CPUOFF);        // Exit LPM0
	}	
}
