//
//	This file contains all of the code to read, decode, and process the events
//	contained in the node EEPROM
//


#include "event.h"
#include "OneWireMaster.h"

//
//	GetNodeEvent
//
//	Fills an event dtruct with the next event. If it sees something
//	that isn't an event, it'll puke.
//
//	This routine requires that the required device is active.
//

void GetNodeEvent(struct EventStruct *event){
	unsigned char	byte;			// a byte of data;
		
	byte = OWReadByte();			// first byte of event entry
	event->time = (byte << 8);		// save first byte
	if (byte >= 0xFD) 				// We have an escape, extention, or EOF
		return;
		
	byte = OWReadByte();			// second byte, LSB of time
	event->time = (event->time | (((unsigned short) byte) && 0x00ff));
	
	event->type = OWReadByte();		// what should we do?
	event->thing = OWReadByte();	// what do we modify
	
	return;
}

//
//	ProcessEvent
//
//	Do what ever the event says to do.
//
//	Currently, the "nodeID" is unused, since we don't know how to return to where we were, 
//	if we change the node.
//

void ProcessEvent(unsigned char nodeID, struct EventStruct *event){
	unsigned char ccb;				// channel control byte
	
	if (event->time > 0xfd00)		// not a real event?
		return;

	// determine if we're changing P0 or P1
	//
	if (event->thing == 0)
		ccb = 0x04;
	else
		ccb = 0x08;
		
	OWWriteByte(0xF0);				// update channel control
	OWWriteByte(ccb);
	OWWriteByte(0xff);				// reserved
	
	switch (event->type) {
		case eTURNON:
			OWWriteBit(1);	
		case eTURNOFF:
			OWWriteBit(0);
	}
}

